/*
   This file is part of KDE/aRts (Noatun) - xine integration
   Copyright (C) 2002 Ewald Snel <ewald@rambo.its.tudelft.nl>
   Copyright (C) 2002 Neil Stevens <neil@qualityassistant.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License version 2 as published by the Free Software Foundation.
*/

#ifndef ARTSKDE_KVIDEOWIDGET_H
#define ARTSKDE_KVIDEOWIDGET_H

#include <qevent.h>
#include <qimage.h>
#include <qvaluevector.h>
#include <qwidget.h>
#include <kmedia2.h>
#include <kxmlguiclient.h>


/**
 * Video widget class.
 */
class KVideoWidget : public QWidget, virtual public KXMLGUIClient
{
Q_OBJECT

public:
    /**
     * Constructor.
     */
    KVideoWidget( KXMLGUIClient *clientParent, QWidget *parent = 0, const char *name = 0, WFlags f = 0 );

    /**
     * Constructor.
     */
    KVideoWidget( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );

    /**
     * Destructor.
     */
    virtual ~KVideoWidget();

    /**
     * Embed and display video on this widget.
     * @param vpo the PlayObject to embed
     */
    void embed( Arts::VideoPlayObject vpo );

    /**
     * Snapshot.
     */
    static QImage snapshot( Arts::VideoPlayObject vpo );

    /**
     * Returns true if this widget is currently displaying a video.
     */
    bool isEmbedded();

    /**
     * Returns true if the video is shown in full screen.
     */
    bool isFullscreen();

    /**
     * Returns true if the video is shown at half size (50%).
     */
    bool isHalfSize();

    /**
     * Returns true if the video is shown at normal size (100%).
     */
    bool isNormalSize();

    /**
     * Returns true if the video is shown at double size (200%).
     */
    bool isDoubleSize();

    /**
     * Returns the recommended size for the widget.
     */
    QSize sizeHint() const;

    /**
     * Returns the preferred height for this widget, given the width w.
     */
    virtual int heightForWidth ( int w ) const;

    /**
     * Get language by ISO-631 language code.
     * @param code the language code
     * @return the language in English
     */
    QString getLanguageByCode( const QString &code );

    /**
     * Returns the current audio channel.
     */
    int getAudioChannel();

    /**
     * Returns a list of available audio languages.
     * Empty strings indicate the language of the channel is unknown.
     */
    QValueVector<QString> getAudioLanguages();

    /**
     * Returns the current subtitle channel or -1 if subtitles are
     * disabled.
     */
    int getSubtitleChannel();

    /**
     * Returns a list of available subtitle languages.
     * Empty strings indicate the language of the channel is unknown.
     */
    QValueVector<QString> getSubtitleLanguages();

protected:
    virtual void mousePressEvent( QMouseEvent *event );
    virtual void resizeEvent( QResizeEvent *event );
    virtual bool x11Event( XEvent *event );

public slots:
				void clickRightButton( const QPoint &p );
    /**
     * Select full screen mode.
     */
    void setFullscreen();

    /**
     * Select windowed mode.
     */
    void setWindowed();

    /**
     * Select half (50%) video size, emits an 'adaptSize()' signal.
     */
    void setHalfSize();

    /**
     * Select normal (100%) video size, emits an 'adaptSize()' signal.
     */
    void setNormalSize();

    /**
     * Select double (200%) video size, emits an 'adaptSize()' signal.
     */
    void setDoubleSize();

    /**
     * Select audio channel.
     * @param channel the audio channel
     */
    void setAudioChannel( int channel );

    /**
     * Select subtitle channel.
     * @param channel the subtitle channel or -1 to disable subtitles
     */
    void setSubtitleChannel( int channel );

    /**
     * Video display size change notification.
     * @param width the new video display width
     * @param height the new video display height
     */
    void resizeNotify( int width, int height );

protected slots:
    void fullscreenActivated();
    void halfSizeActivated();
    void normalSizeActivated();
    void doubleSizeActivated();

signals:
    /**
     * Adapt GUI to new video size.
     * @param width the suggested width for this widget
     * @param height the suggested height for this widget
     */
    void adaptSize( int width, int height );

    /**
     * Right mouse button pressed, 
     * @param p the location of the mouse cursor
     */
    void rightButtonPressed( const QPoint &p );

    /**
     * Current audio channel or PlayObject has changed.
     * @param channel the active audio channel
     */
    void audioChannelChanged( int channel );

    /**
     * List of audio languages or PlayObject has changed.
     */
    void audioLanguagesChanged();

    /**
     * Current subtitle channel or PlayObject has changed.
     * @param channel the active subtitle channel
     */
    void subtitleChannelChanged( int channel );

    /**
     * List of subtitle languages or PlayObject has changed.
     */
    void subtitleLanguagesChanged();

private:
    void init(void);
    QWidget *fullscreenWidget;
    int videoWidth;
    int videoHeight;
    Arts::VideoPlayObject poVideo;
};

#endif
